'use strict';

var path = require('path');
var src = path.resolve.bind(path, __dirname, 'templates');
var isValid = require('is-valid-app');

module.exports = function(app, base) {
  if (!isValid(app, 'generate-project')) return;
  var dest = app.options.dest || app.cwd;
  var task = createTask(app, dest);

  /**
   * Plugins (register other generators as plugins)
   */

  app.use(require('generate-collections'));
  app.use(require('generate-defaults'));
  app.use(require('generate-editorconfig'));
  app.use(require('generate-eslint'));
  app.use(require('generate-git'));
  app.use(require('generate-license'));
  app.use(require('generate-package'));
  app.use(require('generate-travis'));
  app.use(require('generate-install'));

  /**
   * Scaffold out a basic node.js project.
   *
   * ```sh
   * $ gen project
   * ```
   * @name project
   * @api public
   */

  task('project', ['files'], 'index.js');

  /**
   * Scaffold out a basic [gulp][] plugin project.
   *
   * ```sh
   * $ gen project:gulp-plugin
   * # also aliased as
   * $ gen project:gp
   * ```
   * @name project:gulp-plugin
   * @api public
   */

  app.task('gp', ['gulp-plugin']);
  task('gulp-plugin', ['files'], 'gulp-plugin.js');

  /**
   * Scaffold out a basic [base][] plugin project.
   *
   * ```sh
   * $ gen project:base-plugin
   * # also aliased as
   * $ gen project:bp
   * ```
   * @name project:base-plugin
   * @api public
   */

  app.task('bp', ['base-plugin']);
  task('base-plugin', ['files'], 'base-plugin.js');

  /**
   * Scaffold out a minimal code project,
   *
   * ```sh
   * $ gen project:min
   * # also aliased as
   * $ gen project:minimal
   * ```
   * @name project:min
   * @api public
   */

  app.task('min', ['minimal']);
  app.task('minimal', ['package', 'license', 'gitignore', 'gitattributes']);

  /**
   * Scaffold out a standard code project,
   *
   * ```sh
   * $ gen project:standard
   * # this is also the default task, and can be run with the following
   * $ gen project
   * ```
   * @name project:standard
   * @api public
   */

  app.task('standard', ['minimal', 'editorconfig', 'verbmd']);

  /**
   * Scaffold out a basic [generate][] generator project.
   *
   * ```sh
   * $ gen project:generator
   * # also aliased as
   * $ gen project:gen
   * ```
   * @name project:generator
   * @api public
   */

  task('gen', ['files'], 'generator/*.js');

  /**
   * Single-file generators
   */

  app.task('files', [
    'editorconfig',
    'eslint',
    'eslintignore',
    'gitattributes',
    'gitignore',
    'travis',
    'package',
    'license'
  ]);

  /**
   * Generates a basic node.js project, then prompts to install dependencies
   * after files are written.
   *
   * ```sh
   * $ gen project:generator
   * # also aliased as
   * $ gen project:gen
   * ```
   * @name project:generator
   * @api public
   */

  app.task('default', ['project']);
};

/**
 * Create a task with the given `name` and glob `pattern`
 */

function createTask(app, dest) {
  return function(name, dependencies, pattern) {
    app.task(name, dependencies, function() {
      return app.src(src(pattern))
        .pipe(app.renderFile('*'))
        .pipe(app.conflicts(dest))
        .pipe(app.dest(dest));
    });
  };
}
